import { useMemo, ReactElement } from 'react';
import { NodeAttribute, QueryGraphEdges, SchemaReactflowEntityNode, handleDataFromReactflowToDataId, toHandleId } from '../../model';
import { Handle, Position } from 'reactflow';
import { Abc, CalendarToday, Map, Numbers, Place, QuestionMarkOutlined } from '@mui/icons-material';
import Icon from '@graphpolaris/shared/lib/components/icon';
import { PillHandle } from '@graphpolaris/shared/lib/components/pills/PillHandle';
import { pillDropdownPadding } from '@graphpolaris/shared/lib/components/pills/pill.const';

type PillDropdownProps = {
  node: SchemaReactflowEntityNode;
  attributes: NodeAttribute[];
  attributeEdges: (QueryGraphEdges | undefined)[];
  hovered: boolean;
  handleBeingDraggedIdx: number;
  onHandleMouseDown: (attribute: NodeAttribute, i: number, event: React.MouseEvent) => void;
};

type IconMapType = {
  [key: string]: ReactElement;
};

const IconMap: IconMapType = {
  temporal: <CalendarToday />,
  spatial: <Map />,
  numerical: <Numbers />,
  categorical: <Abc />,
  default: <Place />,
};

export const PillDropdown = (props: PillDropdownProps) => {
  const forceOpen = true;
  const openNumbers: number[] = useMemo(() => {
    if (forceOpen || props.hovered) return props.attributes.map((_, i) => i);

    //collect connected nodes
    const connected = props.attributes
      .map((attribute) => {
        const nodeID = toHandleId(handleDataFromReactflowToDataId(props.node, attribute));
        return props.attributeEdges.some((edge) => edge?.sourceHandleData && toHandleId(edge?.sourceHandleData) === nodeID);
      })
      .map((connected, i) => (connected ? i : -1))
      .filter((i) => i !== -1);

    // add to it the node being dragged
    if (props.handleBeingDraggedIdx !== -1 && !connected.includes(props.handleBeingDraggedIdx)) connected.push(props.handleBeingDraggedIdx);
    return connected.sort();
  }, [props]);

  return (
    <>
      <div
        className={
          'border-[1px] border-secondary-200 divide-y divide-secondary-200 ' +
          (openNumbers.length > 0 ? 'animate-openmenu ' : 'animate-closemenu ')
        }
      >
        {openNumbers.map((i) => {
          const attribute = props.attributes[i];
          if (attribute.handleData.attributeName === undefined) {
            throw new Error('attribute.handleData.attributeName is undefined');
          }

          return (
            <div
              className="px-2 py-1 bg-secondary-100 flex justify-between items-center"
              key={(attribute.handleData.attributeName || '') + i}
              onMouseDown={(event: React.MouseEvent) => {
                props.onHandleMouseDown(attribute, i, event);
              }}
            >
              <p className="truncate text-[0.6rem]">{attribute.handleData.attributeName}</p>
              {attribute.handleData?.attributeDimension && <Icon component={IconMap[attribute.handleData.attributeDimension]} size={16} />}
              <PillHandle
                mr={-pillDropdownPadding}
                handleTop="auto"
                position={Position.Right}
                className={'fill-accent-500 stroke-white'}
                type="square"
              >
                <Handle
                  id={toHandleId(handleDataFromReactflowToDataId(props.node, attribute))}
                  type="source"
                  position={Position.Right}
                  className={'!rounded-none !bg-transparent !w-full !h-full !right-0 !left-0 !border-0'}
                ></Handle>
              </PillHandle>
            </div>
          );
        })}
      </div>
    </>
  );
};
