import React from 'react';
import { CompositeLayer } from 'deck.gl/typed';
import { ScatterplotLayer } from '@deck.gl/layers/typed';
import { getProperty } from '../../utlis';
import * as d3 from 'd3';
import NodeOptions from './NodeOptions';
import { Node, LayerProps } from '../../types';

export const NodeLayerConfig = {
  colisionFilter: true,
  opacity: 1,
  fillColor: {
    filled: true,
    type: 'rgb',
    label: 'color',
    accessor: 'getFillColor',
    fixed: true,
    defaultValue: [255, 0, 0],
    value: [255, 0, 0],
    scaleRange: [
      [255, 0, 0],
      [0, 0, 255],
    ],
    condition: '',
  },
};

export class NodeLayer extends CompositeLayer<LayerProps> {
  static type = 'Node';
  static layerOptions = NodeLayerConfig;

  static generateLayerOptions(layer: any, updatedLayer: void, graphInfo: { [key: string]: any }, deleteLayer: void) {
    return <NodeOptions layer={layer} updatedLayer={updatedLayer} graphInfo={graphInfo} deleteLayer={deleteLayer} />;
  }

  updateState({ props, oldProps, context, changeFlags }: { props: any; oldProps: any; context: any; changeFlags: any }) {
    const { graph, config } = props;

    if (!config.fillColor.fixed) {
      const colorAttribute = graph.getGraphInfo().nodeAttributes[config.fillColor.condition];
      const colorScale = this.createColorScale(config.fillColor, colorAttribute);
      this.setState({ colorScale });
    }
  }

  createColorScale(colorProp: { [key: string]: any }, colorAttribute: { [key: string]: any }) {
    console.log(colorAttribute);
    if (colorAttribute.dataType == 'boolean') {
      return d3.scaleOrdinal().domain(['0', '1']).range(colorProp.scaleRange);
    } else if (colorAttribute.dataType == 'number') {
      return d3.scaleLinear().domain(colorAttribute.values).range(colorProp.scaleRange);
    } else if (colorAttribute.dataType == 'string') {
      return d3
        .scaleOrdinal()
        .domain([...colorAttribute.values])
        .range(colorProp.scaleRange);
    } else {
      return null;
    }
  }

  getColor(node: Node, colorProps: { [key: string]: any }) {
    if (colorProps.fixed) {
      return colorProps.value !== undefined ? colorProps.value : colorProps.defaultValue;
    } else {
      const { condition } = colorProps;
      if (!condition) {
        return colorProps.defaultValue;
      }
      const value = getProperty(node.attributes, condition);
      return this.state.colorScale(value);
    }
  }

  renderLayers() {
    const { graph, config, visible } = this.props;

    return new ScatterplotLayer({
      hidden: visible,
      data: graph.getNodes(),
      pickable: true,
      opacity: config.opacity,
      filled: config.fillColor.filled,
      radiusScale: 6,
      radiusMinPixels: 7,
      radiusMaxPixels: 100,
      lineWidthMinPixels: 1,
      getPosition: (d: any) => [d.attributes.long, d.attributes.lat],
      getFillColor: (d: any) => this.getColor(d, config.fillColor),
      getRadius: (d: any) => 5,
    });
  }
}

NodeLayer.layerName = 'Node';
